#include <arpa/inet.h>
#include <assert.h>
#include <ctype.h>
#include <fcntl.h>
#include <limits.h>
#include <math.h>
#include <pwd.h>
#include <signal.h>
#include <string.h>
#include <stdarg.h>
#include <sys/dir.h>
#include <sys/ioctl.h>
#include <sys/resource.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <syslog.h>
#include <utime.h>
#include <time.h>   // Needed for radutmp.h
#include "radutmp.h"
#include "common.h"

#define	USAGE_TOTAL		0
#define	USAGE_DAY		1
#define	USAGE_WEEK		2
#define	USAGE_MONTH		3
#define	USAGE_YEAR		4

void                     update(word myHostAddr, time_t t, int effect, int loggingLevel);

int			 sendMail(UserRec *ur, char *login, char *cc, char *subject, char *sPath, char *dPath);

int			 warnExpire(time_t expire);
void			 writeUsageRecords(int curMinute);
void			 unlinkOldLockFiles();
void                     WarnExpire(UserRec *ur, char *login);

// *************************************************************************

int
main(int argc, char **argv)
{
  int                      i,
                           automatic = 1,     // do nightly maint.
                           runAsDaemon = 1,   // run as a daemon.
                           effect = 1,        // if 0, dont do anything.
                           loggingLevel = 1;  // more verbose logging.
  struct stat              _stat;
  struct tm               *tmP;
  struct in_addr           _in_addr;
  char                     hostname[256];
  time_t                   t,
                           configTime;
  word                     myHostAddr;
  char			  *_argv[16] = { "acua", NULL, NULL };
  char			   arg[16];

  readConfig();
  configTime = time(NULL);
  
  while ((i = getopt(argc, argv, "adel")) != EOF) {
    switch (i) {
    case 'a':
      automatic = 0;
      break;
    case 'd':
      runAsDaemon = 0;
      break;
    case 'e':
      effect = 0;
      break;
    case 'l':
      loggingLevel = atoi(argv[optind++]);
      break;
    }
  }

  openlog("acua_radupdated", LOG_PID, LOG_DAEMON);
  
  gethostname(hostname, 256);
  myHostAddr = hostAddr(hostname);
  
  _in_addr.s_addr = myHostAddr;
  if (loggingLevel >= 1)
    syslog(LOG_INFO, "hostname = %s, IP = %s",
           hostname,
           inet_ntoa(_in_addr));

  // announce the modes acua is running in.
  syslog(LOG_INFO, "Current settings:");
  if (automatic) {
    syslog(LOG_INFO, "Maint: Yes");
  } else {
    syslog(LOG_INFO, "Maint: No");
  }
  if (runAsDaemon) {
    syslog(LOG_INFO, "Daemon: Yes");
  } else {
    syslog(LOG_INFO, "Daemon: No");
  }
  if (effect) {
    syslog(LOG_INFO, "Effect: Yes");
  } else {
    syslog(LOG_INFO, "Effect: No");
  }
  syslog(LOG_INFO, "Logging Level: %d", loggingLevel);

  if (runAsDaemon) daemonInit();

  for (;;) {
    // re-read the config file if it's been updated
    if (stat(configFilePath, &_stat) < 0)
      perrQuit("couldn't stat %s", configFilePath);
    if (_stat.st_mtime > configTime) {
      readConfig();
      if (configTime && loggingLevel >= 1)
        syslog(LOG_INFO, "re-read %s", configFilePath);
      configTime = _stat.st_mtime;
    }

    // remove any old lock files
    time(&t);
    tmP = localtime(&t);
    if (tmP->tm_min == 0) {
      unlinkOldLockFiles();
    }
    
    // "acua expire"
    if (automatic && (tmP->tm_hour == 0) && (tmP->tm_min == 1))
	acua_expire(1, _argv);
    
    // do "acua sync", "acua purge" and "acua expire"
    if (automatic && (tmP->tm_hour == 3)) {
      switch (tmP->tm_min) {
      case 0:
        // sync
        acua_sync(1, _argv);
        break;
      case 1:
        // purge
        if (optPurgeDays >= 14) {
          sprintf(arg, "%d", optPurgeDays);
          _argv[1] = arg;
          acua_purge(2, _argv);
          _argv[1] = NULL;
        }
        break;
//      case 2:
        // expire
//        acua_expire(1, _argv);
//        break;
      }
    }

    // do the update
    update(myHostAddr, t, effect, loggingLevel);
    if (userFile != NULL) {
      userFileClose();
      syslog(LOG_ERR, "Coding error, userbase had to be closed after update()");
    }

    // back to sleep.. yawn.
    time(&t);
    tmP = localtime(&t);
    sleep(60 - tmP->tm_sec);
  }
  return(0);
}

void
update(word myHostAddr, time_t t, int effect, int loggingLevel)
{
  char			   login[MAX_LOGINCHARS	+ 1],
			   dPath[256],
                           subject[64];
  int			   i,
			   sLeft,
			   curClass,
			   deductMins,
			   deductTime,
			   deductSession;
  UserRec		   ur;
  struct radutmp	   rt;
  struct in_addr           __in_addr, *ina;
  int			   hostIdx=0;
  // data accountancy section variables
//  int                      detailOpen = 0, res = 0;
//  char                     session_id[9];
//  radacct                  acct;
  

  // FIXME: new code to implement 2 NAS's, will implmement much better later
  struct { int a;  // kick needs to exec for this nas (0=no, 1=yes);
           int b;  // kick needs to exec for this nas (0=no, 1=yes);
           unsigned int a_ip;  // ip addr of NAS a
           char a_pass[33];    // password for NAS a
           unsigned int b_ip;  // ip addr of NAS b
           char b_pass[33];    // password for NAS b
           FILE *a_fp; // pointer to be used to open command file for NAS.
           FILE *b_fp; // pointer to be used to open command file for NAS.
           char a_nastalk[80]; // filename of the command file for nastalk.
           char b_nastalk[80]; // filename of the command file for nastalk.
           char a_exec[200]; // command line to execute
           char b_exec[200]; // command line to execute
         } naskick;

  ina = &__in_addr;
  inet_aton("203.34.65.249", ina);  // IP Address of Portmaster.
  naskick.a_ip = ina->s_addr;
  inet_aton("203.34.65.209", ina);  // IP Address of Portslave box.
  naskick.b_ip = ina->s_addr;
  naskick.a = 0;
  naskick.b = 0;

  userFileOpen();

  while	(!userFileRead(&ur)) {
    // make sure the user has a /etc/passwd entry
    if (!loginFromUID(login, ur.uid)) continue;
    // find the index of our host
    hostIdx = findHost(&ur, myHostAddr);
    // warn the user of account expiry.
    WarnExpire(&ur, login);
    // are they online?
    if (ur.nHosts == 0) continue;
    // Ok. We think they're logged in and we're allowed to tinker with 'em.
    // increment tMinutes for this user
    ur.tMinutes++;
    // See if the user is in radutmp, if not then mark them as logged out.
    if (!radiusCheckOnline(&rt, login)) {
      // We were wrong, the user isn't online.

      // remove this host from the users account.
      removeHost(&ur, myHostAddr);
      ur.nHosts = 0;  // dirty hack.
      // update the users acua record..
      if (effect) userFileEdit(&ur);
      // log that we marked them as offline if we actually did (effect)
      if ((loggingLevel >= 1) && (effect))
        syslog(LOG_ERR, "Marked [%s] as logged off (not online).", login);
      continue;
    }
    // this user is online.
    
    if (OVERTIME(ur.flagz) && ((ur.expire - time(NULL)) / 86400) < 1)
    {
    	// turn off the overtime feature as there is not going to be enough
    	// subscription days left to do overtime.
    	ur.flagz &= ~FLG_OVERTIME;
    	
    	// send a email.
    	snprintf(subject, sizeof(subject)-1, "Overtime: Subscription too short.");
    	snprintf(dPath, sizeof(dPath)-1, "/tmp/acua_updated.XXXXXX");
    	mktemp(dPath);
    	sendMail(&ur, login, "", subject, LIB"/acua_updated.overtime_shortsub", dPath);
    }
    
    // Deduct the appropriate amount of time.
    deductMins = 1;
    deductTime = deductSession = 1;
    // get current time class
    curClass = curTimeClass(&ur);
    for (i = 0; i < nTimeClasses; i++) {
      if (inTimeClass(i)) {
        if (ur.cLimit[i] <= -3) deductTime = 0;
        if (ur.cLimit[i] <= -2) deductSession = 0;
        if (ur.cLimit[i] >= 0 && (!TCSMARTTIME(ur.flags)))
          ur.cLeft[i] -= deductMins;
      }
    }
    if (deductTime) ur.tLeft -= deductMins;
    if (deductSession) ur.sLeft -= deductMins;
    // update the users record to say when they were last online.
    time(&ur.lastOnline[hostIdx]);
    // figure out how much time they have left
    sLeft = INT_MAX;
    if (ur.tLimit >= 0) sLeft = min(sLeft, ur.tLeft + ur.credit);
    if (ur.sLimit >= 0) sLeft = min(sLeft, ur.sLeft);
    if (curClass >= 0 && ur.cLimit[curClass] >= 0)
      sLeft = min(sLeft, ur.cLeft[curClass]);
    // send a warning message if necessary (about time limit exceeding)
    if (effect && WARNBOOT(ur.flags)) {
      if (!OVERTIME(ur.flagz) && (optPPPWarnBoot > 0) && (sLeft == optPPPWarnBoot) && (ur.sLimit < 1440)) {
        // set the subject
        snprintf(subject, sizeof(subject)-1, "Warning: possible disconnection within %d minutes", optPPPWarnBoot);
        // set tempory file template
        snprintf(dPath, sizeof(dPath)-1, "/tmp/acua_updated.XXXXXX");
        // make the tempory file
        mktemp(dPath);
        // send the e-mail
        sendMail(&ur, login, "", subject, LIB"/acua_updated.warn_boot", dPath);
      }

      if (OVERTIME(ur.flagz) && (optPPPWarnBoot > 0) && (sLeft == optPPPWarnBoot) && (ur.sLimit < 1440)) {
        // set the subject
        snprintf(subject, sizeof(subject)-1, "Warning: OVERTIME will activate in %d minutes", optPPPWarnBoot);
        // set tempory file template
        snprintf(dPath, sizeof(dPath)-1, "/tmp/acua_updated.XXXXXX");
        // make the tempory file
        mktemp(dPath);
        // send the e-mail
        sendMail(&ur, login, "", subject, LIB"/acua_updated.warn_overtime", dPath);
      }
    }
    // is it time for this user to be kicked off?

    /* NEW CODE, OVERTIME option */
    if ((sLeft <= 0) && (OVERTIME(ur.flagz)))
    {
	syslog(LOG_ERR, "Activated OVERTIME feature for [%s]", login);

	/* make sure they have a day left to knock off (so we dont run into
	 * negative expiry dates) */
	if (((ur.expire - time(NULL)) / 86400) > 1)
	{
	    syslog(LOG_ERR, "%s", "Looks ok, more than a day left.");
	    // deduct a day
	    ur.expire -= 86400;
	    // Fix time and session remaining.
	    ur.tLeft += 60;
	    ur.sLeft += 60;
	    sLeft += 60;
	    
    	    snprintf(subject, sizeof(subject)-1, "Overtime: New expiry date notification.");
    	    snprintf(dPath, sizeof(dPath)-1, "/tmp/acua_updated.XXXXXX");
    	    mktemp(dPath);
    	    sendMail(&ur, login, "", subject, LIB"/acua_updated.overtime_newexp", dPath);
	}
    }
    /* NEW CODE, OVERTIME function END */

    if (sLeft > 0) {
      if (effect) userFileEdit(&ur);  // Nope
      continue;
    }
    // remove us from the users list of hosts that he/she is logged into
    // we do this here instead of below because that way if there's a problem
    // below we wont go getting into a stupid endless loop.
    removeHost(&ur, myHostAddr);
    // update the users account to reflect the changes
    if (effect) userFileEdit(&ur);
    if (loggingLevel >= 1)
      syslog(LOG_INFO, "Marked [%s] as logged out.", login);
    // FIXME: This will fuck up in MultiLink or when maxLogins > 1 and more
    //        than one connection is active, or maybe it wont cause of the
    //        while loop we're actually in at this very moment (not the one
    //        below!).  Study code and fix either this comment or the code.
    //
    // FIXME: from now on the code that handles the NAS's is absolute
    //        shit.  It needs a re-think and a re-write something shocking.
    //
    // Disconnect the user
    // Make the command file for nastalk
    if ((rt.nas_address == naskick.a_ip) && (naskick.a!=1)) {
      // user is on nas 'a'.
      naskick.a = 1;  // yes, we need to execute nastalk.
      __in_addr.s_addr = rt.nas_address;
      snprintf(naskick.a_nastalk, sizeof(naskick.a_nastalk), "/tmp/acua_nastalk.%s.XXXXXX", inet_ntoa(__in_addr));
      mktemp(naskick.a_nastalk);
      if ((naskick.a_fp = fopen(naskick.a_nastalk, "w")) == NULL) {
        perrQuit("Couldn't create tempory file (%s)", naskick.a_nastalk);
      }
      // set password to login with.
      strncpy(naskick.a_pass, "PUT_NAS_!ROOT_PASSWORD_HERE", sizeof(naskick.a_pass));
      #ifdef PM3  // or PortSlave box
        fputs("set     timeout 10\n", naskick.a_fp);
        fputs("expect  \"ogin:\"\n", naskick.a_fp);
        fputs("send    \"!root\\r\"\n", naskick.a_fp);
        fputs("expect  \"word:\"\n", naskick.a_fp);
        fprintf(naskick.a_fp, "send    \"%s\\r\"\n", naskick.a_pass);
        fputs("expect  \">\"\n", naskick.a_fp);
      #else
        // Assume this is one of them AWEFUL Shiva LanRovers.
        fputs("set     timeout 10\n", naskick.a_fp);
        fputs("expect  \"id:\"\n", naskick.a_fp);
        fputs("send    \"root\\r\"\n", naskick.a_fp);
        fputs("expect  \"word?\"\n", naskick.a_fp);
        fprintf(naskick.a_fp, "send    \"%s\\r\"\n", naskick.a_pass);
        fputs("expect  \"#\"\n", naskick.a_fp);
      #endif
    }

    if ((rt.nas_address == naskick.b_ip) && (naskick.b!=1)) {
      // user is on nas 'b'.
      naskick.b = 1;  // yes, we need to execute nastalk.
      __in_addr.s_addr = rt.nas_address;
      snprintf(naskick.b_nastalk, sizeof(naskick.b_nastalk), "/tmp/acua_nastalk.%s.XXXXXX", inet_ntoa(__in_addr));
      mktemp(naskick.b_nastalk);
      if ((naskick.b_fp = fopen(naskick.b_nastalk, "w")) == NULL) {
        syslog(LOG_ERR, "Cant open the tempory file %s", naskick.b_nastalk);
        fclose(naskick.b_fp);
        continue;
      }
      // set password to login with.
      strncpy(naskick.b_pass, "PUT_NAS_!ROOT_PASSWORD_HERE", sizeof(naskick.b_pass));
      #ifdef PM3  // or PortSlave box
        fputs("set     timeout 10\n", naskick.b_fp);
        fputs("expect  \"ogin:\"\n", naskick.b_fp);
        fputs("send    \"!root\\r\"\n", naskick.b_fp);
        fputs("expect  \"word:\"\n", naskick.b_fp);
        fprintf(naskick.b_fp, "send    \"%s\\r\"\n", naskick.b_pass);
        fputs("expect  \">\"\n", naskick.b_fp);
      #else
        // Assume this is one of them AWEFUL Shiva LanRovers.
        fputs("set     timeout 10\n", naskick.b_fp);
        fputs("expect  \"id:\"\n", naskick.b_fp);
        fputs("send    \"root\\r\"\n", naskick.b_fp);
        fputs("expect  \"word?\"\n", naskick.b_fp);
        fprintf(naskick.b_fp, "send    \"%s\\r\"\n", naskick.b_pass);
        fputs("expect  \"#\"\n", naskick.b_fp);
      #endif
    }

    // check to see if this user is on a nas that we know about.
    if ((naskick.a!=1) && (naskick.b!=1)) {
      if (loggingLevel >= 1)  // nope.
        syslog(LOG_INFO, "Source code error - didn't match NAS - cant kick user(s).");
      continue;
    }
    // so far so good
    if (loggingLevel >= 1)
      syslog(LOG_INFO, "Marked [%s|%s|s%d] for kicking.", login, inet_ntoa(__in_addr), rt.nas_port);
    // write hang-up commands into the tempory file for "expect" to execute.
    if (rt.nas_address == naskick.a_ip) {
      #ifdef PM3  // or PortSlave box
        fprintf(naskick.a_fp, "send    \"reset s%d\\r\"\n", rt.nas_port);
        fputs("expect  \"port S\"\n", naskick.a_fp);
      #else
        // Assume we're a Shiva piece of crap
        fprintf(naskick.a_fp, "send    \"clear call-interface %d\\r\"\n", rt.nas_port);
        fputs("expect  \"?\"\n", naskick.a_fp);
        fputs("send    \"y\\r\"\n", naskick.a_fp);
        fputs("expect  \"#\"\n", naskick.a_fp);
      #endif
    }
    // write hang-up commands into the tempory file for "expect" to execute.
    if (rt.nas_address == naskick.b_ip) {
      #ifdef PM3  // or PortSlave box
        fprintf(naskick.b_fp, "send    \"reset s%d\\r\"\n", rt.nas_port);
        fputs("expect  \"port S\"\n", naskick.b_fp);
      #else
        // Assume we're a Shiva piece of crap
        fprintf(naskick.b_fp, "send    \"clear call-interface %d\\r\"\n", rt.nas_port);
        fputs("expect  \"?\"\n", naskick.b_fp);
        fputs("send    \"y\\r\"\n", naskick.b_fp);
        fputs("expect  \"#\"\n", naskick.b_fp);
      #endif
    }
  }  // end of _very_ big while loop
  // close the acua userbase file
  userFileClose();
  // are we needing to kick anyone off?
  if ((naskick.a!=1) && (naskick.b!=1)) return;  // Nope
  // finish off the code for the "expect" program
  if (naskick.a == 1) {
    #ifdef PM3
      fputs("send    \"exit\\r\"\n", naskick.a_fp);
    #else
      fputs("send    \"quit\\r\"\n", naskick.a_fp);
    #endif
    fclose(naskick.a_fp);
    __in_addr.s_addr = naskick.a_ip;
    snprintf(naskick.a_exec, sizeof(naskick.a_exec)-1, "%s %s %s", PATH_TO_NASTALK, inet_ntoa(__in_addr), naskick.a_nastalk);
    if (loggingLevel >= 1)
      syslog(LOG_INFO, "Executing nastalk to kick users, nas: %s", inet_ntoa(__in_addr));
    if (system(naskick.a_exec)) {
      // probably an error logging the user off.
      syslog(LOG_ERR, "!! ERROR: Problem running nastalk, nas: %s", inet_ntoa(__in_addr));
      syslog(LOG_ERR, "Command: (%s)", naskick.a_exec);
    }
    /*
    if ((unlink(naskick.a_nastalk)) && (loggingLevel >= 1))
      syslog(LOG_ERR, "!! ERROR: Could not delete tempory file: %s", naskick.a_nastalk);
    */
  }

  // finish off the code for the "expect" program
  if (naskick.b == 1) {
    #ifdef PM3
      fputs("send    \"exit\\r\"\n", naskick.b_fp);
    #else
      fputs("send    \"quit\\r\"\n", naskick.b_fp);
    #endif
    fclose(naskick.b_fp);
    __in_addr.s_addr = naskick.b_ip;
    snprintf(naskick.b_exec, sizeof(naskick.b_exec)-1, "%s %s %s", PATH_TO_NASTALK, inet_ntoa(__in_addr), naskick.b_nastalk);
    if (loggingLevel >= 1)
      syslog(LOG_INFO, "Executing nastalk to kick users, nas: %s", inet_ntoa(__in_addr));
    if (system(naskick.b_exec)) {
      // probably an error logging the user off.
      syslog(LOG_ERR, "!! ERROR: Problem running nastalk, nas: %s", inet_ntoa(__in_addr));
      syslog(LOG_ERR, "Command: (%s)", naskick.b_exec);
    }
    /*
    if ((unlink(naskick.b_nastalk)) && (loggingLevel >= 1))
      syslog(LOG_ERR, "!! ERROR: Could not delete tempory file: %s", naskick.b_nastalk);
    */
  }
  syslog(LOG_INFO, "end of while loop");
}


int
sendMail(UserRec *ur, char *login, char *cc, char *subject, char *sPath, char *dPath)
{
  int   cstat, timer;
  pid_t cpid;
  char  *dest;
  
  // make the mail to send the user, from the template file in sPath
  preprocessFile(ur, sPath, dPath);
  // take care of where this mail is going
  if (optMailHost[0] != '\0') {
    dest = (char *) malloc(strlen(optMailHost)+strlen(login)+2);
    if (!dest) errQuit("Not enough memory to hold users email address");
    sprintf(dest, "%s@%s", login, optMailHost);
  } else {
    dest = (char *) malloc(strlen(login)+1);
    if (!dest) errQuit("Not enough memory to hold users email address");
    sprintf(dest, "%s", login);
  }
  
  // fork a new process
  cpid = fork();
  
  if (cpid == 0) {
    // child process - replace stdin with the output from preprocessFile()
    // BTW - changing the uid etc after the freopen is not a bug.
    freopen(dPath, "r", stdin);
    // change to a different user if need be
    if (optMailUser) {
      setuid(optMailUser);   seteuid(optMailUser);
    }
    // change to different group if need be
    if (optMailGroup) {
      setgid(optMailGroup);  setegid(optMailGroup);
    }

    // send the email
    if (cc == NULL) execl(optMailProg, "mail", "-s", subject, dest, NULL); else
	execl(optMailProg, "mail", "-s", subject, "-c", cc, dest, NULL);
    // FIXME: what if execl fails!
  } else if (cpid == -1) {
    syslog(LOG_ERR, "Warning - Could not send mail to [%s], fork failed.", login);
    unlink(dPath);
    // free the malloc'd memory.
    free(dest);
    return(1);
  }
  // this is a safety measure, just incase the mail prog crashes.
  timer = time(NULL);
  for (;;) {
    if (waitpid(cpid, &cstat, WNOHANG) > 0) break;
    if (time(NULL) > timer+10) {
      syslog(LOG_ERR, "Warning - [%s] crashed. Killing dead process", optMailProg);
      kill(cpid, SIGKILL); // go away
      // free the malloc'd memory.
      free(dest);
      return(1);
    }
    usleep(250);
  }
  // free the malloc'd memory.
  free(dest);
  // delete the tempory file we made.
  unlink(dPath);
  return(0);
}


/* void
sendMail(UserRec *ur, char *login, char *cc, char *subject, char *sPath, char *dPath)
{
  char mailDest[64];
  char str[64];

  // set the mail destination
  strcpy(mailDest, login);
  if (optMailHost[0])
    sprintf(mailDest + strlen(mailDest), "@%s", optMailHost);
  preprocessFile(ur, sPath, dPath);
  sprintf(str, "mail -s \"%s\" %s < %s", subject, mailDest, dPath);
  system(str);
  unlink(dPath);
} */

// warn user that they will be expiring soon.
int
warnExpire(time_t expire) {
  int			   i;
  time_t		   t = time(NULL), low,	high;

  for (i = 0; i	< nExpireWarnTimes; i++) {
    low	= expire - expireWarnTime[i] * 24 * 60 * 60;
    high = expire - (expireWarnTime[i] - 1) * 24 * 60 *	60;
    if (t >= low + 120 && t < high) return 1;
  }
  return 0;
}

void
unlinkOldLockFiles()
{
  DIR		 *dir =	opendir(LIB);
  char		  path[256];
  struct dirent	 *_dirent;
  struct stat	  _stat;

  if (dir == NULL)
    perrQuit("opendir(%s)", LIB);

  while	((_dirent = readdir(dir)) != NULL) {
    if (!strncmp(_dirent->d_name, "LOCK", 4)) {
      sprintf(path, "%s/%s", LIB, _dirent->d_name);
      if (lstat(path, &_stat))
	continue;
      if (time(NULL) - _stat.st_atime >= 3600)
	unlink(path);
    }	   
  }
  closedir(dir);
}

void WarnExpire(UserRec *ur, char *login) {
  char subject[64];
  char cc[64] = "";
  char dPath[256];

  // warn users when their accounts/subscriptions are almost expired
  if (ur->expire) {
    if (WARNEXPIRE(ur->flags) && !warnExpire(ur->expire)) {
      ur->flags &= ~FLG_WARNEXPIRE;
      userFileEdit(ur);
    } else
    if (!WARNEXPIRE(ur->flags) && warnExpire(ur->expire)) {
      // set the subject
      if (EXPIRE(ur->flags) == EXPIRE_DELETE)
	strcpy(subject, "Account Expires Soon");
      else
	strcpy(subject, "Subscription Expires Soon");
      // set the CC string
      if (optWarnExpireCC[0] != '\0') {
        sprintf(cc, "-c %s ", optWarnExpireCC);
      }
      // send the e-mail
      strcpy(dPath, "/tmp/acua_updated.XXXXXX");
      mktemp(dPath);
      sendMail(ur, login, cc, subject, LIB"/acua_updated.warn_expire", dPath);
      ur->flags |= FLG_WARNEXPIRE;
      userFileEdit(ur);
    }
  }
}

